#' @title Example of 'LRErdd' in shiny version
#' @description With this function a shiny app will be opened with all the functionalities of 'LRErdd'
#' @return  A shiny app will be opened
#' @export
#' @examples
#' if (interactive()) {
#'   open_LRErdd_framework()
#' }
open_LRErdd_framework <- function() {
    appDir <- system.file("shiny-examples", "LRErdd", package = "LRErdd")
    if (appDir == "") {
        stop("Could not find example directory. Try re-installing `LRErdd`.", call. = FALSE)
    }
    
    shiny::runApp(appDir, display.mode = "normal")
}
