# Variables used in non-standard evaluation (ggplot2 aesthetics and data
# manipulation inside methods). Declared here to avoid spurious
# "no visible binding for global variable" notes in R CMD check.
utils::globalVariables(c(
  "forcingvalues", "density", "Var1", "Var2", "Freq", "Dataset",
  "Independent", "Dependent", "Bandwidth", "ACE", "L", "U", "Estimand",
  "ve", "ge", "obs", "izen"
))
