% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TwoSided.R
\name{mcmc.gauss2}
\alias{mcmc.gauss2}
\title{Posterior median of the Complier Average Causal Effect (CACE) for Gaussian outcomes}
\usage{
mcmc.gauss2(n.iter, n.burn, dat)
}
\arguments{
\item{n.iter}{A positive integer specifying the number of iterations (including burn-in)}

\item{n.burn}{A positive integer specifying the number of burn-in iterations}

\item{dat}{Data frame}
}
\value{
The value of the posterior median of CACE for Gaussian outcomes
}
\description{
mcmc.gauss2() uses the Markov Chain Monte Carlo (MCMC) algorithm with the Data Augmentation (DA) method to derive
the posterior median of CACE for normally distributed outcomes in the presence of two-sided noncompliance under 
monotonicity (no defiers) and exclusion restrictions for both never-takers and always-takers
}
