% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RegDis_function.r
\name{fuzzy_fep_bw}
\alias{fuzzy_fep_bw}
\title{Bandwidth selection for Fuzzy - FEP bandwidth}
\usage{
fuzzy_fep_bw(
  dataset,
  forcing_var_name = "S",
  Y_name = "dropout",
  niter = 1000,
  W_name = "W",
  bandwidth = c(500, 1000, 1500),
  cut_value = 15000,
  M2 = 5,
  whichunder = 1,
  typemod = "binary",
  typesided = "onesided"
)
}
\arguments{
\item{dataset}{The data frame with the variables}

\item{forcing_var_name}{forcing_var_name}

\item{Y_name}{A character format of the name of the outcome in the dataset}

\item{niter}{Number of iteration to calculate the adjusted p-value.}

\item{W_name}{The name that defines the treatment receipt status.}

\item{bandwidth}{A numerical vector defining the ranges to subset the dataset and create the subpopulations of interest.}

\item{cut_value}{Threshold value to separate the treatment status.}

\item{M2}{number of iterations}

\item{whichunder}{If the treatment status is lower (0) or higher (1) than the threshold}

\item{typemod}{If the outcome is binary or numerical}

\item{typesided}{If the treatment receipt status is onesided or twosided}
}
\value{
data frame with variable and value and bandwidth
}
\description{
fuzzy_fep() was defined to calculate just for one dataset. For the cases where the user wants to test several bandwidths this functions is applied.
}
