% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grants.r
\docType{data}
\name{grants}
\alias{grants}
\title{Italian university grants and student dropout}
\format{
A data frame with 15984 rows (students) and 21 columns:
\describe{
  \item{S}{Forcing variable: measure of the family's economic situation (in euros); students with \code{S} below 15,000 are eligible for the grant}
  \item{Z}{Eligibility indicator (1 = eligible, i.e. \code{S} below the threshold; 0 = not eligible)}
  \item{A}{Grant application indicator (1 = applied)}
  \item{W}{Treatment received: grant receipt indicator (1 = received the grant)}
  \item{dropout}{Outcome: 1 if the student dropped out, 0 otherwise}
  \item{sex}{Student's sex (binary indicator)}
  \item{HSHumanity}{High school type: humanities (binary indicator)}
  \item{HSScience}{High school type: science (binary indicator)}
  \item{HSTech}{High school type: technical (binary indicator)}
  \item{HSOther}{High school type: other (binary indicator)}
  \item{hsgrade}{High school grade}
  \item{Y2004}{Cohort indicator: year 2004}
  \item{Y2005}{Cohort indicator: year 2005}
  \item{Y2006}{Cohort indicator: year 2006}
  \item{University}{University indicator}
  \item{Humanity}{Field of study: humanities (binary indicator)}
  \item{Science}{Field of study: science (binary indicator)}
  \item{Social.Science}{Field of study: social science (binary indicator)}
  \item{BioMed}{Field of study: biology and medicine (binary indicator)}
  \item{Tech}{Field of study: technical (binary indicator)}
  \item{Other}{Field of study: other (binary indicator)}
}
}
\usage{
data(grants)
}
\description{
Data from an Italian study concerning the effects of university grants on
student dropout (Li, Mattei and Mealli, 2015). Eligibility for the grant is
determined by a measure of the economic situation of the student's family
falling below a threshold of 15,000 euros, which makes it a natural
regression discontinuity design.
}
\references{
Li, F., Mattei, A. and Mealli, F. (2015). Bayesian inference for
regression discontinuity designs with application to the evaluation of
Italian university grants. \emph{The Annals of Applied Statistics}, 9(4),
1906-1931. \doi{10.1214/15-AOAS809}
}
\keyword{datasets}
