% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RegDis_function.r
\name{Tave}
\alias{Tave}
\title{Absolute difference in means between treatment groups}
\usage{
Tave(x, z)
}
\arguments{
\item{x}{A numeric vector (covariate values)}

\item{z}{A binary vector with the treatment assignment indicator}
}
\value{
A numeric value: the absolute difference in means between the two groups.
}
\description{
Internal helper that computes the absolute value of the
difference in means of a variable between the two assignment groups. It is
used as test statistic in the randomization-based balance tests.
}
\keyword{internal}
