% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TwoSided.R
\name{EM.bin2}
\alias{EM.bin2}
\title{Maximum Likelihood Estimate (MLE) of the Complier Average Causal Effect (CACE) for binary outcomes}
\usage{
EM.bin2(dat, tol = 1e-04, maxit = 1000)
}
\arguments{
\item{dat}{Data frame}

\item{tol}{The convergence tolerance. 
Convergence is reached when the maximum absolute difference between the parameter values on consecutive step is not greater than @tol.
Defaults to 1e-04}

\item{maxit}{The maximum number of iterations. The default is 1000}
}
\value{
The value of the MLE of CACE for binary outcomes (at either convergence or the last iteration if the maximum number of iterations has been reached before convergence)
}
\description{
EM.bin2() uses the Expectation-Maximization (EM) algorithm to calculate the MLEs of CACE for binary outcomes 
in the presence of two-sided noncompliance under monotonicity (no defiers) and exclusion restrictions for both never-takers and always-takers
}
