% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RegDis_function.r
\name{sharp_neyman}
\alias{sharp_neyman}
\title{SHARP RDD: RANDOMIZATION-BASED INFERENCE - NEYMAN APPROACH}
\usage{
sharp_neyman(Y, Z, cin)
}
\arguments{
\item{Y}{A numerical vector with the data about the outcome}

\item{Z}{forcing binary variable}

\item{cin}{Alpha level for the confidence intervals (e.g. the default 0.05 yields 95\% confidence intervals)}
}
\value{
A vector with values for the effect and the confidence intervals.
}
\description{
sharp_neyman() implements the Neyman approach (Neyman 1923, 1990) for drawing inference on causal effects in the dataset
}
\details{
When the Neyman approach is used for inference, the function calculate the estimate of the average causal effect in this Equation (1) derived using the estimator proposed by Neyman,that is, the difference between the observed averages for units exposed to treatment and units exposed to control.
}
