% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OneSided.R
\name{EM.gauss}
\alias{EM.gauss}
\title{Maximum Likelihood Estimate (MLE) of the Complier Average Causal Effect (CACE) for Gaussian outcomes}
\usage{
EM.gauss(dat, tol = 1e-04, maxit = 1000)
}
\arguments{
\item{dat}{Data frame}

\item{tol}{The convergence tolerance
Convergence is reached when the maximum absolute difference between the parameter values on consecutive step is not greater than @tol
The default is 1e-04}

\item{maxit}{The maximum number of iterations. The default is 1000}
}
\value{
The value of the MLE of CACE for continuous outcomes (at either convergence or the last iteration if the maximum number of iterations has been reached before convergence)
}
\description{
EM.gauss() uses the Expectation-Maximization (EM) algorithm to calculate the MLEs of CACE for continuous normally distributed outcomes 
in the presence of one-sided noncompliance under exclusion restriction for never-takers
}
