% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OneSided.R
\name{da.gauss.h0}
\alias{da.gauss.h0}
\title{Data Augmentation (DA) step of Markov Chain Monte Carlo (MCMC) algorithm to derive the posterior median of the 
Complier Average Causal Effect (CACE) for a Gaussian outcome under the sharp null hypothesis of no effect}
\usage{
da.gauss.h0(theta, dat)
}
\arguments{
\item{theta}{A list with the values of the parameters: 
pi.c = proportion of compliers;
mu.c, sigma2.c = means and variances of the Normal distribution of the outcome for compliers;
mu.nt, sigma2.nt = mean and variance of the Normal distribution of the outcome for never-takers}

\item{dat}{Data frame}
}
\value{
A vector with the imputed compliance status for each unit in the data set under the sharp null hypothesis of no effect
}
\description{
da.gauss.h0() implements the Data Augmentation (DA) method to impute the compliance status of each unit 
for a continuous normally distributed outcome in the presence of one-sided noncompliance under exclusion restriction for never-takers and
under the sharp null hypothesis of no effect
}
