% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/open_LRErdd_framework.r
\name{open_LRErdd_framework}
\alias{open_LRErdd_framework}
\title{Example of 'LRErdd' in shiny version}
\usage{
open_LRErdd_framework()
}
\value{
A shiny app will be opened
}
\description{
With this function a shiny app will be opened with all the functionalities of 'LRErdd'
}
\examples{
if (interactive()) {
  open_LRErdd_framework()
}
}
