% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RegDis_function.r
\name{fuzzy_fep_numeric1sided}
\alias{fuzzy_fep_numeric1sided}
\title{Fuzzy - FEP 1 sided numerical}
\usage{
fuzzy_fep_numeric1sided(dataset, Y, W, Z, Y_name, M2 = 10)
}
\arguments{
\item{dataset}{The data frame with the variables}

\item{Y}{Y A numerical vector with outcome information}

\item{W}{A vector that defines the treatment receipt status.}

\item{Z}{A numerical vector with the forcing variable in binary format.}

\item{Y_name}{A character format of the name of the outcome in the dataset}

\item{M2}{number of iterations}
}
\value{
A list with values of pppv, Tsim, Tobs
}
\description{
Fuzzy - FEP 1 sided numerical
}
\details{
This is for the cases where the noncompliance is two sided and numerical
}
