% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/practical_functions.r
\name{cutting_range}
\alias{cutting_range}
\title{Selection of the cutting range}
\usage{
cutting_range(forcingvalues, threshold, typerange, range_value)
}
\arguments{
\item{forcingvalues}{a vector with forcing variables that will be used to defined the cutting range}

\item{threshold}{a numeric value that define the threshold to create the two groups}

\item{typerange}{a character value that will define the type of selection that we want to carry out. It can be 'percentage', 'balunit' or 'unbalunit'. In the case or percentage, the reduction that we want to do is defined by the proportion of all the range. Instead of proportion, if we want to reduce by an accurate number we can use the other two options. In the case of balunit, the selection will be balanced to the two sides of the threshold. In the case of unbalunit, the selection can be unbalanced and we will need to give two numerical values in the range_value parameter.}

\item{range_value}{a numerical (when typerange is percentage or balunit) and two numerica values (in the case of unbalunit) that define the value to create the range.}
}
\value{
a list with two values that define the lower and upper values to create the selection of the data
}
\description{
cutting_range() defines the values of the forcing values taking into account the threshold, the type of selection and the range value.
}
