% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TwoSided.R
\name{E.step.bin2}
\alias{E.step.bin2}
\title{Expectation step of the Expectation-Maximization (EM) algorithm to calculate the MLE of the Complier Average Causal Effect (CACE) for binary outcomes}
\usage{
E.step.bin2(theta, dat)
}
\arguments{
\item{theta}{A list with the values of the parameters: 
pi.c = proportion of compliers;
py.c0, py.c1 = probabilities that the outcome takes on value 1 for compliers under control and under treatment;
py.nt = probability that the outcome takes on value 1 for never-takers;
py.at = probability that the outcome takes on value 1 for always-takers}

\item{dat}{Data frame}
}
\value{
A vector with the expected compliance status for each unit in the data set
}
\description{
E.step.bin2() implements the Expectation step of the EM algorithm to calculate the MLE of CACE for binary outcomes 
in the presence of two-sided noncompliance under monotonicity (no defiers) and exclusion restrictions for both never-takers and always-takers
}
