% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OneSided.R
\name{E.step.gauss}
\alias{E.step.gauss}
\title{Expectation step of the Expectation-Maximization (EM) algorithm to calculate the MLE of the Complier Average Causal Effect (CACE) for Gaussian outcomes}
\usage{
E.step.gauss(theta, dat)
}
\arguments{
\item{theta}{A list with the values of the parameters: 
pi.c = proportion of compliers;
mu.c0, sigma2.c0, mu.c1, sigma2.c1 = means and variances of the Normal distributions of the outcome for compliers under control and under treatment;
mu.nt, sigma2.nt = mean and variance of the Normal distribution of the outcome for never-takers}

\item{dat}{Data frame}
}
\value{
A vector with the expected compliance status for each unit in the data set
}
\description{
E.step.gauss() implements the Expectation step of the EM algorithm to calculate the MLE of CACE for continuous normally distributed outcomes 
in the presence of one-sided noncompliance under exclusion restriction for never-takers
}
