% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RegDis_function.r
\name{sharp_fep}
\alias{sharp_fep}
\title{Calculate Fisher Exact p-value}
\usage{
sharp_fep(dataset, forcing_bin_var_name = "Z", Y_name = "dropout", niter = 100)
}
\arguments{
\item{dataset}{The data frame with the variables}

\item{forcing_bin_var_name}{The name of the forcing variable in a binary format in the dataset.}

\item{Y_name}{A character format of the name of the outcome in the dataset}

\item{niter}{Number of iteration to calculate the adjusted p-value.}
}
\value{
A data frame the correponding effect of the selection.
}
\description{
sharp_fep implements the Fisher exact p?\210?value approach (Fisher 1925) for drawing inference on causal effects in the dataset
}
\details{
This approach is randomization-based mode of inference because it assumes randomized experiments: it treats the potential outcome as fixed unknown quantities, viewing the assignment variable, Si, and thus, treatment assignment, Zi, as the only source of randomness. Randomization-based modes of inference, not relying on asymptotic approximations, are particularly attractive in RD settings where the analysis may depend on small samplesize. We implement the Fisher exactp?\210?value approach focusing on assessing the sharp (exact) null hypothesis of no effect of the treatment whatsoever, that is, the null hypothesis under which,for each unit in theUs0, both values of the potential outcomes are identical.
}
\examples{
 
\donttest{
data(grants)
sharp_fep(dataset = grants, forcing_bin_var_name = 'Z', 
Y_name = 'dropout', niter = 100)
}
}
