% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RegDis_function.r
\name{fuzzy_neyman}
\alias{fuzzy_neyman}
\title{Fuzzy - neyman approach}
\usage{
fuzzy_neyman(Y, Wh, Z, cin = 0.05)
}
\arguments{
\item{Y}{A numerical vector with outcome information}

\item{Wh}{A vector that defines the treatment receipt status.}

\item{Z}{A numerical vector with the forcing variable in binary format}

\item{cin}{Alpha level for the confidence intervals (e.g. the default 0.05 yields 95\% confidence intervals)}
}
\value{
A vector with information about the effects and confidence intervals.
}
\description{
fuzzy_neyman() calculates the causal effect with the fuzzy neyman approach
}
\details{
The focus will be on the sharp nullhypothesis that disallows any effect of assignment to treatment versus control for compliersbelonging toUs0:H0:Yi(0) =Yi(1)for alli?\210\210 Us0withGi=c, which can be interpreted asthe null hypothesis of no effect of ?\200?receipt?\200\235 of treatment on outcome for the subpopulationof compliers belonging toUs.
}
