% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TwoSided.R
\name{mcmc.gauss.h02}
\alias{mcmc.gauss.h02}
\title{Posterior distributions of the parameter vector and the compliance status of each unit for a Gaussian outcome 
under the sharp null hypothesis of no effect}
\usage{
mcmc.gauss.h02(n.iter, n.burn, dat)
}
\arguments{
\item{n.iter}{A positive integer specifying the number of iterations (including burn-in)}

\item{n.burn}{A positive integer specifying the number of burn-in iterations}

\item{dat}{Data frame}
}
\value{
A list of two elements:
THETA = a matrix with the simulated posterior distribution of the parameter vector for a Gaussian outcome under the sharp null hypothesis of no effect 
Gstatus = a matrix with simulated posterior distribution of the compliance status of each unit under the sharp null hypothesis of no effect
}
\description{
mcmc.gauss.h02() uses the Markov Chain Monte Carlo (MCMC) algorithm with the Data Augmentation (DA) method to derive
the posterior median of CACE for a normally distributed outcome in the presence of two-sided noncompliance 
under monotonicity (no defiers) and exclusion restrictions for both never-takers and always-takers
and under the sharp null hypothesis of no effect
}
