% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RegDis_function.r
\name{rand_pajd}
\alias{rand_pajd}
\title{Randomization-based tests adjusted for multiple testing}
\usage{
rand_pajd(
  dataset,
  forcing_var_name = "S",
  covariates = c("sex", "HSHumanity", "HSTech", "HSOther", "hsgrade", "Y2004"),
  niter = 1000,
  bandwidth = 1000,
  cut_value = 15000,
  whichunder = 1
)
}
\arguments{
\item{dataset}{The data frame with the variables}

\item{forcing_var_name}{The name of the forcing variable in the dataset.}

\item{covariates}{Covariates of the model.}

\item{niter}{Number of iteration to calculate the adjusted p-value.}

\item{bandwidth}{Selected range to subset the dataset and create the subpopulation of interest.}

\item{cut_value}{Threshold value to separate the treatment status.}

\item{whichunder}{If the treatment status is lower (0) or higher (1) than the threshold}
}
\value{
A data frame with the selected variables and the correponding mean difference with adjusted p-value.
}
\description{
rand_pajd() calculates, for each of the selected variables and in the selected subpopulation the meand difference and adjusted p-value.
}
\details{
The table shows the mean difference of each variable between treatment groups, and the corresponding p-value derived using randomization-based tests adjusted for multiple testing.
}
\examples{
 
\donttest{
data(grants)
rand_pajd(dataset = grants, forcing_var_name = 'S', 
covariates = c('sex', 'HSHumanity', 'HSTech', 'HSOther', 
'hsgrade', 'Y2004'), niter = 1000, bandwidth = 1000, 
cut_value = 15000, whichunder = 1)
}
}
