% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RegDis_function.r
\name{fuzzy_neyman_bw}
\alias{fuzzy_neyman_bw}
\title{Bandwidth selection for Fuzzy - neyman bandwidth}
\usage{
fuzzy_neyman_bw(
  dataset,
  forcing_var_name = "S",
  Y_name = "dropout",
  niter = 1000,
  W = "W",
  bandwidth = c(500, 1000, 1500),
  cut_value = 15000,
  whichunder = 1,
  cin = 0.05
)
}
\arguments{
\item{dataset}{The data frame with the variables}

\item{forcing_var_name}{The name of the forcing variable in the dataset.}

\item{Y_name}{A character format of the name of the outcome in the dataset}

\item{niter}{Number of iteration to calculate the adjusted p-value.}

\item{W}{The name that defines the treatment receipt status in the dataset.}

\item{bandwidth}{A numerical vector defining the ranges to subset the dataset and create the subpopulations of interest.}

\item{cut_value}{Threshold value to separate the treatment status.}

\item{whichunder}{If the treatment status is lower (0) or higher (1) than the threshold}

\item{cin}{Alpha level for the confidence intervals (e.g. the default 0.05 yields 95\% confidence intervals)}
}
\value{
A data frame with effect and bandwidths
}
\description{
fuzzy_neyman() was defined to calculate just for one dataset. For the cases where the user wants to test several bandwidths this functions is applied.
}
