% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OneSided.R
\name{M.step.bin}
\alias{M.step.bin}
\title{Maximization step of the Expectation-Maximization (EM) algorithm to calculate the MLE of the Complier Average Causal Effect (CACE) for binary outcomes}
\usage{
M.step.bin(G, dat)
}
\arguments{
\item{G}{A vector with the expected compliance status for each unit in the data set}

\item{dat}{Data frame}
}
\value{
A list with the values of the parameters: 
pi.c = proportion of compliers;
py.c0, py.c1 = probabilities that the outcome takes on value 1 for compliers under control and under treatment;
py.nt = probability that the outcome takes on value 1 for never-takers
}
\description{
M.step.bin() implements the Maximization step of the EM algorithm to calculate the MLE of CACE for binary outcomes 
in the presence of one-sided noncompliance under exclusion restriction for never-takers
}
