% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OneSided.R
\name{plus}
\alias{plus}
\title{Shift probabilities away from the boundaries of the unit interval}
\usage{
plus(x)
}
\arguments{
\item{x}{A numeric vector of probabilities}
}
\value{
A numeric vector of the same length as \code{x} with values moved
slightly away from 0 and 1.
}
\description{
Internal helper that adds (or subtracts) 0.01 to probabilities
equal or close to 0 (or 1) to avoid numerical problems in likelihood
evaluations.
}
\keyword{internal}
