% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/practical_functions.r
\name{all_binom2num}
\alias{all_binom2num}
\title{From binomial to numerical}
\usage{
all_binom2num(data)
}
\arguments{
\item{data}{dataset}
}
\value{
A new dataset. Convert every binomial column in the dataset as numerical to avoid errors
}
\description{
all_binom2num() analizes a data frame, detects the binomial columns and convert them to numeric format.
}
\details{
In the shiny application we need to verify some conditions and this function helps us to check it.
}
\examples{
 
\donttest{
df <- data.frame(
gp = factor(rep(letters[1:3], each = 10)),
y = rnorm(30),
a = sample(c('0','1'), 30, replace = TRUE))

all_binom2num(df)
}
}
