% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TwoSided.R
\name{da.gauss2}
\alias{da.gauss2}
\title{Data Augmentation (DA) step of Markov Chain Monte Carlo (MCMC) algorithm to derive the posterior median of the 
Complier Average Causal Effect (CACE) for Gaussian outcomes}
\usage{
da.gauss2(theta, dat)
}
\arguments{
\item{theta}{A list with the values of the parameters: 
pi.c = proportion of compliers;
mu.c0, sigma2.c0, mu.c1, sigma2.c1 = means and variances of the Normal distributions of the outcome for compliers under control and under treatment;
mu.nt, sigma2.nt = mean and variance of the Normal distribution of the outcome for never-takers;
mu.at, sigma2.at = mean and variance of the Normal distribution of the outcome for always-takers}

\item{dat}{Data frame}
}
\value{
A vector with the imputed compliance status for each unit in the data set
}
\description{
da.gauss2() implements the Data Augmentation (DA) method to impute the compliance status of each unit 
for normally distributed outcomes in the presence of two-sided noncompliance under 
monotonicity (no defiers) and exclusion restrictions for both never-takers and always-takers
}
