% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OneSided.R
\name{da.bin.h0}
\alias{da.bin.h0}
\title{Data Augmentation (DA) step of Markov Chain Monte Carlo (MCMC) algorithm to derive the posterior median of the 
Complier Average Causal Effect (CACE) for a binary outcome under the sharp null hypothesis of no effect}
\usage{
da.bin.h0(theta, dat)
}
\arguments{
\item{theta}{A list with the values of the parameters: 
pi.c = proportion of compliers;
py.c = probability that the outcome takes on value 1 for compliers;
py.nt = probability that the outcome takes on value 1 for never-takers}

\item{dat}{Data frame}
}
\value{
A vector with the imputed compliance status for each unit in the data set under the sharp null hypothesis of no effect
}
\description{
da.bin.h0() implements the Data Augmentation (DA) method to impute the compliance status of each unit 
for binary outcomes in the presence of one-sided noncompliance under exclusion restriction for never-takers and
under the sharp null hypothesis of no effect
}
